﻿using CarpentryWorkshopContracts.BindingModels;
using CarpentryWorkshopContracts.BusinessLogicsContracts;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace CarpentryWorkshopView
{
    public partial class FormWoodSale : Form
    {
        private readonly ILogger _logger;

        private readonly IWoodLogic _logicWood;

        private readonly IShopLogic _logicShop;

        public FormWoodSale(ILogger<FormSupply> logger, IWoodLogic logicWood, IShopLogic logicShop)
        {
            InitializeComponent();
            _logger = logger;
            _logicWood = logicWood;
            _logicShop = logicShop;
        }

        private void FormWoodSale_Load(object sender, EventArgs e)
        {
            _logger.LogInformation("Ice creams loading");
            try
            {
                var list = _logicWood.ReadList(null);
                if (list != null)
                {
                    comboBoxWood.DisplayMember = "WoodName";
                    comboBoxWood.ValueMember = "Id";
                    comboBoxWood.DataSource = list;
                    comboBoxWood.SelectedItem = null;
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ice creams loading error");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonSale_Click(object sender, EventArgs e)
        {
            if (comboBoxWood.SelectedValue == null)
            {
                MessageBox.Show("Выберите мороженое", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (string.IsNullOrEmpty(textBoxCount.Text))
            {
                MessageBox.Show("Заполните поле Количество", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _logger.LogInformation("Ice cream sale");
            try
            {
                var operationResult = _logicShop.MakeSell(
                    new WoodBindingModel
                    {
                        Id = Convert.ToInt32(comboBoxWood.SelectedValue)
                    },
                    Convert.ToInt32(textBoxCount.Text)
                );
                if (!operationResult)
                {
                    throw new Exception("Ошибка при продаже.");
                }
                MessageBox.Show("Продажа прошла успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Wood sale error");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
