﻿using CarpentryWorkshopContracts.BindingModels;
using CarpentryWorkshopContracts.SearchModels;
using CarpentryWorkshopContracts.StoragesContracts;
using CarpentryWorkshopContracts.ViewModels;
using CarpentryWorkshopDatabaseImplement.Models;
using CarpentryWorkshopDataModels.Models;
using Microsoft.EntityFrameworkCore;

namespace CarpentryWorkshopDatabaseImplement.Implements
{
	public class ShopStorage : IShopStorage
	{
		public ShopViewModel? Delete(ShopBindingModel model)
		{
			using var context = new CarpentryWorkshopDatabase();
			var element = context.Shops.FirstOrDefault(x => x.Id == model.Id);
			if (element != null)
			{
				context.Shops.Remove(element);
				context.SaveChanges();
				return element.GetViewModel;
			}
			return null;
		}

		public ShopViewModel? GetElement(ShopSearchModel model)
		{
			if (string.IsNullOrEmpty(model.ShopName) && !model.Id.HasValue)
			{
				return null;
			}
			using var context = new CarpentryWorkshopDatabase();
			return context.Shops
				.Include(x => x.Woods)
				.ThenInclude(x => x.Wood)
				.FirstOrDefault(x => model.Id.HasValue && x.Id == model.Id)
				?.GetViewModel;
		}

		public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
		{
			if (string.IsNullOrEmpty(model.ShopName))
			{
				return new();
			}
			using var context = new CarpentryWorkshopDatabase();
			return context.Shops
				.Include(x => x.Woods)
				.ThenInclude(x => x.Wood)
				.Select(x => x.GetViewModel)
				.Where(x => x.ShopName.Contains(model.ShopName ?? string.Empty))
				.ToList();
		}

		public List<ShopViewModel> GetFullList()
		{
			using var context = new CarpentryWorkshopDatabase();
			return context.Shops
			.Include(x => x.Woods)
				.ThenInclude(x => x.Wood)
				.Select(x => x.GetViewModel)
				.ToList();
		}

		public ShopViewModel? Insert(ShopBindingModel model)
		{
			using var context = new CarpentryWorkshopDatabase();
			try
			{
				var newShop = Shop.Create(context, model);
				if (newShop == null)
				{
					return null;
				}
				if (context.Shops.Any(x => x.ShopName == newShop.ShopName))
				{
					throw new Exception("Не должно быть два магазина с одним названием");
				}
				context.Shops.Add(newShop);
				context.SaveChanges();
				return newShop.GetViewModel;
			}
			catch
			{
				throw;
			}
		}

		public ShopViewModel? Update(ShopBindingModel model)
		{
			using var context = new CarpentryWorkshopDatabase();
			var shop = context.Shops.FirstOrDefault(x => x.Id == model.Id);
			if (shop == null)
			{
				return null;
			}
			try
			{
				if (context.Shops.Any(x => (x.ShopName == model.ShopName && x.Id != model.Id)))
				{
					throw new Exception("Не должно быть два магазина с одним названием");
				}
				shop.Update(model);
				shop.UpdateWoods(context, model);
				context.SaveChanges();
				return shop.GetViewModel;
			}
			catch
			{
				throw;
			}
		}
		public bool SellWoods(IWoodModel model, int count)
		{
			if (model == null)
				return false;
			using var context = new CarpentryWorkshopDatabase();
			using var transaction = context.Database.BeginTransaction();
			List<ShopWood> lst = new List<ShopWood>();
			foreach (var el in context.ShopWoods.Where(x => x.WoodId == model.Id))
			{
				int dif = count;
				if (el.Count < dif)
					dif = el.Count;
				el.Count -= dif;
				count -= dif;
				if (el.Count == 0)
				{
					lst.Add(el);
				}
				if (count == 0)
					break;
			}
			if (count > 0)
			{
				transaction.Rollback();
				return false;
			}
			foreach (var el in lst)
			{
				context.ShopWoods.Remove(el);
			}
			context.SaveChanges();
			transaction.Commit();
			return true;
		}
	}
}
