﻿using CarpentryWorkshopContracts.BindingModels;
using CarpentryWorkshopContracts.SearchModels;
using CarpentryWorkshopContracts.StoragesContracts;
using CarpentryWorkshopContracts.ViewModels;
using CarpentryWorkshopDataModels.Models;
using CarpentryWorkshopFileImplement.Models;

namespace CarpentryWorkshopFileImplement.Implements
{
    public class ShopStorage : IShopStorage
    {
        private readonly DataFileSingleton source;

        public ShopStorage()
        {
            source = DataFileSingleton.GetInstance();
        }

        public List<ShopViewModel> GetFullList()
        {
            return source.Shops.Select(x => x.GetViewModel).ToList();
        }

        public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ShopName))
            {
                return new();
            }
            return source.Shops.Where(x => x.ShopName.Contains(model.ShopName)).Select(x => x.GetViewModel).ToList();
        }
        public ShopViewModel? GetElement(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ShopName) && !model.Id.HasValue)
            {
                return null;
            }
            return source.Shops.FirstOrDefault(x =>
            (!string.IsNullOrEmpty(model.ShopName) && x.ShopName == model.ShopName) ||
            (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }

        public ShopViewModel? Insert(ShopBindingModel model)
        {
            model.Id = source.Shops.Count > 0 ? source.Shops.Max(x => x.Id) + 1 : 1;
            var newShop = Shop.Create(model);
            if (newShop == null)
            {
                return null;
            }
            source.Shops.Add(newShop);
            source.SaveShops();
            return newShop.GetViewModel;
        }

        public ShopViewModel? Update(ShopBindingModel model)
        {
            var shop = source.Shops.FirstOrDefault(x => x.Id == model.Id);
            if (shop == null)
            {
                return null;
            }
            shop.Update(model);
            source.SaveShops();
            return shop.GetViewModel;
        }

        public ShopViewModel? Delete(ShopBindingModel model)
        {
            var shop = source.Shops.FirstOrDefault(x => x.Id == model.Id);
            if (shop != null)
            {
                source.Shops.Remove(shop);
                source.SaveShops();
                return shop.GetViewModel;
            }
            return null;
        }

        public bool CheckAvailability(int woodId, int count)
        {
			int minus = source.Shops
                .SelectMany(shop => shop.ShopWoods
				.Where(wood => wood.Value.Item1.Id == woodId)
				.Select(wood => wood.Value.Item2))
				.Sum(); 
            count -= minus;
            return count <= 0;
        }

        public bool SellWoods(IWoodModel model, int count)
        {
            var wood = source.Woods.FirstOrDefault(x => x.Id == model.Id);

            if (wood == null || !CheckAvailability(wood.Id, count))
            {
                return false;
            }

            for (int i = 0; i < source.Shops.Count; i++)
            {
                var shop = source.Shops[i];
                var woods = shop.ShopWoods;

                foreach (var wod in woods.Where(x => x.Value.Item1.Id == wood.Id))
                {
                    var min = Math.Min(wod.Value.Item2, count);
                    woods[wod.Value.Item1.Id] = (wod.Value.Item1, wod.Value.Item2 - min);
                    count -= min;

                    if (count <= 0)
                    {
                        break;
                    }
                }

                shop.Update(new ShopBindingModel
                {
                    Id = shop.Id,
                    ShopName = shop.ShopName,
                    Address = shop.Address,
                    DateOpen = shop.DateOpen,
                    WoodMaxCount = shop.WoodMaxCount,
                    ShopWoods = woods
                });
            }
            source.SaveShops();
            return true;
        }
    }
}
