﻿using CarpentryWorkshopContracts.BindingModels;
using CarpentryWorkshopContracts.SearchModels;
using CarpentryWorkshopContracts.StoragesContracts;
using CarpentryWorkshopContracts.ViewModels;
using CarpentryWorkshopDataModels.Models;
using CarpentryWorkshopListImplement.Models;

namespace CarpentryWorkshopListImplement.Implements
{
    public class ShopStorage : IShopStorage
    {
        private readonly DataListSingleton _source;
        public ShopStorage()
        {
            _source = DataListSingleton.GetInstance();
        }
        public List<ShopViewModel> GetFullList()
        {
            var result = new List<ShopViewModel>();
            foreach (var shop in _source.Shops)
            {
                result.Add(shop.GetViewModel);
            }
            return result;
        }
        public List<ShopViewModel> GetFilteredList(ShopSearchModel
       model)
        {
            var result = new List<ShopViewModel>();
            if (string.IsNullOrEmpty(model.ShopName))
            {
                return result;
            }
            foreach (var shop in _source.Shops)
            {
                if (shop.ShopName.Contains(model.ShopName))
                {
                    result.Add(shop.GetViewModel);
                }
            }
            return result;
        }
        public ShopViewModel? GetElement(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ShopName) && !model.Id.HasValue)
            {
                return null;
            }
            foreach (var shop in _source.Shops)
            {
                if ((!string.IsNullOrEmpty(model.ShopName) &&
               shop.ShopName == model.ShopName) ||
                (model.Id.HasValue && shop.Id == model.Id))
                {
                    return shop.GetViewModel;
                }
            }
            return null;
        }
        public ShopViewModel? Insert(ShopBindingModel model)
        {
            model.Id = 1;
            foreach (var shop in _source.Shops)
            {
                if (model.Id <= shop.Id)
                {
                    model.Id = shop.Id + 1;
                }
            }
            var newShop = Shop.Create(model);
            if (newShop == null)
            {
                return null;
            }
            _source.Shops.Add(newShop);
            return newShop.GetViewModel;
        }
        public ShopViewModel? Update(ShopBindingModel model)
        {
            foreach (var shop in _source.Shops)
            {
                if (shop.Id == model.Id)
                {
                    shop.Update(model);
                    return shop.GetViewModel;
                }
            }
            return null;
        }
        public ShopViewModel? Delete(ShopBindingModel model)
        {
            for (int i = 0; i < _source.Shops.Count; ++i)
            {
                if (_source.Shops[i].Id == model.Id)
                {
                    var element = _source.Shops[i];
                    _source.Shops.RemoveAt(i);
                    return element.GetViewModel;
                }
            }
            return null;
        }
        public bool CheckAvailability(int woodId, int count)
        {
            int minus = _source.Shops.Select(x => x.ShopWoods.Select(y => (y.Value.Item1.Id == woodId ? y.Value.Item2 : 0)).Sum()).Sum();
            count -= minus;
            return count <= 0;
        }

        public bool SellWoods(IWoodModel model, int count)
        {
            var wood = _source.Woods.FirstOrDefault(x => x.Id == model.Id);

            if (wood == null || !CheckAvailability(wood.Id, count))
            {
                return false;
            }

            for (int i = 0; i < _source.Shops.Count; i++)
            {
                var shop = _source.Shops[i];
                var woods = shop.ShopWoods;

                foreach (var wod in woods.Where(x => x.Value.Item1.Id == wood.Id))
                {
                    var min = Math.Min(wod.Value.Item2, count);
                    woods[wod.Value.Item1.Id] = (wod.Value.Item1, wod.Value.Item2 - min);
                    count -= min;

                    if (count <= 0)
                    {
                        break;
                    }
                }

                shop.Update(new ShopBindingModel
                {
                    Id = shop.Id,
                    ShopName = shop.ShopName,
                    Address = shop.Address,
                    DateOpen = shop.DateOpen,
                    WoodMaxCount = shop.WoodMaxCount,
                    ShopWoods = woods
                });
            }
            return true;
        }
    }
}
